# -*- coding: utf-8 -*-

import os
import sys
import tempfile
import shutil
import signal
import mimetypes
from os.path import join, abspath, normpath, isfile, dirname
from subprocess import Popen, PIPE

# XXX apxs2 -q LIBEXECDIR

APACHE_CONF = '''
Listen %(host)s:%(port)d

ServerRoot %(apache_root)s
AcceptMutex flock
LockFile %(apache_root)s/lock
PidFile %(apache_root)s/pid

ServerName vinnetou

<IfModule !cgi_module>
LoadModule cgi_module %(libexecdir)s/mod_cgi.so
</IfModule>
<IfModule !mime_module>
LoadModule mime_module %(libexecdir)s/mod_mime.so
</IfModule>
<IfModule !mime_magic_module>
LoadModule mime_magic_module %(libexecdir)s/mod_mime_magic.so
</IfModule>
<IfModule !alias_module>
LoadModule alias_module %(libexecdir)s/mod_alias.so
</IfModule>
<IfModule !env_module>
LoadModule env_module %(libexecdir)s/mod_env.so
</IfModule>
<IfModule !autoindex_module>
LoadModule autoindex_module %(libexecdir)s/mod_autoindex.so
</IfModule>
<IfModule !log_config_module>
LoadModule log_config_module %(libexecdir)s/mod_log_config.so
</IfModule>
<IfModule !auth_basic_module>
LoadModule auth_basic_module %(libexecdir)s/mod_auth_basic.so
</IfModule>
<IfModule !authz_default_module>
LoadModule authz_default_module %(libexecdir)s/mod_authz_default.so
</IfModule>
<IfModule !rewrite_module>
LoadModule rewrite_module %(libexecdir)s/mod_rewrite.so
</IfModule>

RewriteEngine on
RewriteCond   %%{HTTP:Authorization} ^(.*)
RewriteRule   ^(.*) - [E=HTTP_AUTHORIZATION:%%1]

#<Files ~ "^\.ht">
#    Order allow,deny
#    Deny from all
#</Files>
DefaultType text/plain
HostnameLookups Off

Alias /icons/ "/usr/share/apache2/icons/"


TypesConfig %(mime_types)s

AddType application/x-compress .Z
AddType application/x-gzip .gz .tgz
AddType application/x-bzip2 .bz2

AddLanguage ca .ca
AddLanguage cs .cz .cs
AddLanguage da .dk
AddLanguage de .de
AddLanguage el .el
AddLanguage en .en
AddLanguage eo .eo
AddLanguage es .es
AddLanguage et .et
AddLanguage fr .fr
AddLanguage he .he
AddLanguage hr .hr
AddLanguage it .it
AddLanguage ja .ja
AddLanguage ko .ko
AddLanguage ltz .ltz
AddLanguage nl .nl
AddLanguage nn .nn
AddLanguage no .no
AddLanguage pl .po
AddLanguage pt .pt
AddLanguage pt-BR .pt-br
AddLanguage ru .ru
AddLanguage sv .sv
AddLanguage tr .tr
AddLanguage zh-CN .zh-cn
AddLanguage zh-TW .zh-tw
AddCharset us-ascii    .ascii .us-ascii
AddCharset ISO-8859-1  .iso8859-1  .latin1
AddCharset ISO-8859-2  .iso8859-2  .latin2 .cen
AddCharset ISO-8859-3  .iso8859-3  .latin3
AddCharset ISO-8859-4  .iso8859-4  .latin4
AddCharset ISO-8859-5  .iso8859-5  .cyr .iso-ru
AddCharset ISO-8859-6  .iso8859-6  .arb .arabic
AddCharset ISO-8859-7  .iso8859-7  .grk .greek
AddCharset ISO-8859-8  .iso8859-8  .heb .hebrew
AddCharset ISO-8859-9  .iso8859-9  .latin5 .trk
AddCharset ISO-8859-10  .iso8859-10  .latin6
AddCharset ISO-8859-13  .iso8859-13
AddCharset ISO-8859-14  .iso8859-14  .latin8
AddCharset ISO-8859-15  .iso8859-15  .latin9
AddCharset ISO-8859-16  .iso8859-16  .latin10
AddCharset ISO-2022-JP .iso2022-jp .jis
AddCharset ISO-2022-KR .iso2022-kr .kis
AddCharset ISO-2022-CN .iso2022-cn .cis
AddCharset Big5        .Big5       .big5 .b5
AddCharset cn-Big5     .cn-big5
AddCharset WINDOWS-1251 .cp-1251   .win-1251
AddCharset CP866       .cp866
AddCharset KOI8      .koi8
AddCharset KOI8-E      .koi8-e
AddCharset KOI8-r      .koi8-r .koi8-ru
AddCharset KOI8-U      .koi8-u
AddCharset KOI8-ru     .koi8-uk .ua
AddCharset ISO-10646-UCS-2 .ucs2
AddCharset ISO-10646-UCS-4 .ucs4
AddCharset UTF-7       .utf7
AddCharset UTF-8       .utf8
AddCharset UTF-16      .utf16
AddCharset UTF-16BE    .utf16be
AddCharset UTF-16LE    .utf16le
AddCharset UTF-32      .utf32
AddCharset UTF-32BE    .utf32be
AddCharset UTF-32LE    .utf32le
AddCharset euc-cn      .euc-cn
AddCharset euc-gb      .euc-gb
AddCharset euc-jp      .euc-jp
AddCharset euc-kr      .euc-kr
AddCharset EUC-TW      .euc-tw
AddCharset gb2312      .gb2312 .gb
AddCharset iso-10646-ucs-2 .ucs-2 .iso-10646-ucs-2
AddCharset iso-10646-ucs-4 .ucs-4 .iso-10646-ucs-4
AddCharset shift_jis   .shift_jis .sjis
AddHandler type-map var
AddType text/html .shtml
AddOutputFilter INCLUDES .shtml

IndexOptions FancyIndexing VersionSort HTMLTable NameWidth=*
#DescriptionCharset=UTF-8
AddIconByEncoding (CMP,/icons/compressed.gif) x-compress x-gzip x-bzip2
AddIconByType (TXT,/icons/text.gif) text/*
AddIconByType (IMG,/icons/image2.gif) image/*
AddIconByType (SND,/icons/sound2.gif) audio/*
AddIconByType (VID,/icons/movie.gif) video/*
AddIcon /icons/binary.gif .bin .exe
AddIcon /icons/binhex.gif .hqx
AddIcon /icons/tar.gif .tar
AddIcon /icons/world2.gif .wrl .wrl.gz .vrml .vrm .iv
AddIcon /icons/compressed.gif .Z .z .tgz .gz .zip
AddIcon /icons/a.gif .ps .ai .eps
AddIcon /icons/layout.gif .html .shtml .htm .pdf
AddIcon /icons/text.gif .txt
AddIcon /icons/c.gif .c
AddIcon /icons/p.gif .pl .py
AddIcon /icons/f.gif .for
AddIcon /icons/dvi.gif .dvi
AddIcon /icons/uuencoded.gif .uu
AddIcon /icons/script.gif .conf .sh .shar .csh .ksh .tcl
AddIcon /icons/tex.gif .tex
AddIcon /icons/bomb.gif /core
AddIcon (SND,/icons/sound2.gif) .ogg
AddIcon (VID,/icons/movie.gif) .ogm
AddIcon /icons/back.gif ..
AddIcon /icons/hand.right.gif README
AddIcon /icons/folder.gif ^^DIRECTORY^^
AddIcon /icons/blank.gif ^^BLANKICON^^
AddIcon /icons/odf6odt-20x22.png .odt
AddIcon /icons/odf6ods-20x22.png .ods
AddIcon /icons/odf6odp-20x22.png .odp
AddIcon /icons/odf6odg-20x22.png .odg
AddIcon /icons/odf6odc-20x22.png .odc
AddIcon /icons/odf6odf-20x22.png .odf
AddIcon /icons/odf6odb-20x22.png .odb
AddIcon /icons/odf6odi-20x22.png .odi
AddIcon /icons/odf6odm-20x22.png .odm
AddIcon /icons/odf6ott-20x22.png .ott
AddIcon /icons/odf6ots-20x22.png .ots
AddIcon /icons/odf6otp-20x22.png .otp
AddIcon /icons/odf6otg-20x22.png .otg
AddIcon /icons/odf6otc-20x22.png .otc
AddIcon /icons/odf6otf-20x22.png .otf
AddIcon /icons/odf6oti-20x22.png .oti
AddIcon /icons/odf6oth-20x22.png .oth
DefaultIcon /icons/unknown.gif
ReadmeName README.html
HeaderName HEADER.html
IndexIgnore .??* *~ *# RCS CVS *,v *,t


Timeout 300
KeepAlive On
MaxKeepAliveRequests 100
KeepAliveTimeout 15

DefaultType text/plain
HostnameLookups Off

LogLevel debug
LogFormat "%%h %%l %%u %%t \\"%%r\\" %%>s %%b" common
CustomLog %(apache_root)s/log common
ErrorLog %(apache_root)s/log

#<IfModule mod_ssl>
#    Listen 8443
#</IfModule>

%(aliases)s
DocumentRoot        %(apache_root)s
ScriptAliasMatch    ^.*$                   %(apache_root)s/wsgi
SetEnv              SCRIPT_ALIAS_MATCH     /

'''


CGI = '''\
#!%(executable)s

import os
from paste.deploy import loadapp
from wsgiref.handlers import CGIHandler
from vinnetou.utils import sane_environ

sane_environ(os.environ)
app = loadapp('config:%(ini)s')
CGIHandler().run(app)
'''


def run_apache_cgi(
    wsgi_app, global_conf, port='8080', host='0.0.0.0', apache_root=None,
    aliases=None,
):

    p = Popen(['whereis', '-b', 'apxs', 'apxs2'], stdout=PIPE)
    out, err = p.communicate()
    apxs = {}
    for l in out.splitlines():
        ver, path = l.split(':', 1)
        path = path.split()
        if path:
            apxs[ver] = path[0]
    apxs = apxs.get('apxs2', apxs.get('apxs'))
    if not apxs:
        print >>sys.stderr, 'apxs not found'
        os._exit(1)

    p = Popen([apxs, '-q', 'LIBEXECDIR'], stdout=PIPE)
    out, err = p.communicate()
    libexecdir = out.strip()

    p = Popen([apxs, '-q', 'SBINDIR'], stdout=PIPE)
    out, err = p.communicate()
    sbindir = out.strip()

    p = Popen([apxs, '-q', 'TARGET'], stdout=PIPE)
    out, err = p.communicate()
    target = out.strip()
    apache_bin = join(sbindir, target)
    if not isfile(apache_bin):
        apache_bin = None

    port = int(port)
    if apache_root is None:
        apache_root = tempfile.mkdtemp(prefix='apache_cgi_')
        is_temp = True
    else:
        is_temp = False
    ini = normpath(abspath(global_conf['__file__']))
    executable = sys.executable

    mime_types = '/etc/mime.types'
    for f in mimetypes.knownfiles:
        if isfile(f):
            mime_types = f
            break

    print
    print 'URL                http://127.0.0.1:%s/' % port
    print
    print 'python executable  %s' % executable
    print 'apxs               %s' % apxs
    print 'apache binary      %s' % apache_bin
    print 'apache root        %s' % apache_root
    print 'apache listen      %s:%s' % (host, port)
    print 'apache libexec dir %s' % libexecdir
    print 'mime types         %s' % mime_types
    print

    if aliases:
        alias_conf = []
        for a in aliases.split():
            p,d = a.split('=',1)
            if ':' in d:
                m,s = d.split(':', 1)
                exec 'import %s' % m
                d = join(abspath(normpath(dirname(eval('%s.__file__' % m)))), s)
            print 'Alias %s %s' % (p, d)
            alias_conf.append('Alias "%s" "%s"' % (p, d))
        aliases = '\n'.join(alias_conf)
    else:
        aliases = ''
    print

    try:
        conf = join(apache_root, 'apache.conf')
        fd = open(conf, 'w')
        fd.write(APACHE_CONF % locals())
        fd.close()

        cgi = join(apache_root, 'wsgi')
        fd = open(cgi, 'w')
        fd.write(CGI % locals())
        fd.close()
        os.chmod(cgi, 0700)

        log = join(apache_root, 'log')
        os.mkfifo(log, 0640)
        try:
            p = Popen([apache_bin, '-f', conf, '-DFOREGROUND'])
            lfd = open(log, 'rt')
            while p.poll() is None:
                d = lfd.readline()
                print d.rstrip()
            lfd.close()
        finally:
            pidfile = join(apache_root, 'pid')
            if isfile(pidfile):
                pid = open(pidfile).read().strip()
                os.kill(int(pid), signal.SIGTERM)
    finally:
        if is_temp:
            shutil.rmtree(apache_root, ignore_errors=True)
