import sys
import optparse
import gc
import os
from cStringIO import StringIO
from wsgiref import handlers

from vinnetou.base import make_wsgi_app
from vinnetou.utils import *


def get_opts():
    parser = optparse.OptionParser(usage='''\
usage: %prog [options] APPMOD URL1 [URL2 ...]

EXAMPLE

    Home page
    $ web4 myapp

    Some URL with default SERVER_NAME=localhost
    $ web4 myapp /shop/category/

    Some URL with explicit SERVER_NAME and SERVER_PORT
    $ web4 myapp http://www.myapp.cz:81/shop/category/

    Profiling
    $ web4 -P /tmp/myapp.prof -I 100 myapp /shop/ /shop/category/ /
    $ prof4 /tmp/myapp.prof

    Vypis sitemapy
    $ web4 --sitemap myapp

'''
    )
    parser.add_option(
        '-S','--print-headers', action='store_true', default=False,
    )
    parser.add_option(
        '-N', '--no-save', action='store_true', default=False,
    )
    parser.add_option(
        '-I', '--iterations', type='int', metavar='NUM', default=1,
    )
    parser.add_option(
        '-C', '--concurrency', type='int', metavar='NUM', default=1,
    )
    parser.add_option(
        '--concurrency-model', metavar='thread/process', default='process',
    )
    parser.add_option(
        '-A', '--http-auth', metavar='USER:PASSWD',
    )
    parser.add_option(
        '-P', '--profile', metavar='FILE',
    )
    parser.add_option(
        '-V', '--var', metavar='VAR=VALUE', action='append',
    )
    parser.add_option(
        '-U', '--upload', metavar='VAR=FILENAME', action='append',
    )
    parser.add_option(
        '-M', '--method', metavar='REQUEST_METHOD',
    )
    parser.add_option(
        '-F', '--form-data', metavar='FORM_DATA',
    )
    parser.add_option(
        '--throw-errors', action='store_true', default=False,
    )
    parser.add_option(
        '--debug-memory', action='store_true', default=False,
    )
    parser.add_option(
        '--rusage', metavar="FILE",
    )
    parser.add_option(
        '--interactive-loop', action='store_true', default=False,
    )
    opts, args = parser.parse_args()
    if not args:
        parser.error('App module required')
    opts.app_name = os.environ['APP_NAME'] = args[0]
    opts.urls = args[1:] or ['http://localhost/']
    return opts


def make_environ(opts, url):
    opts.method
    opts.form_data
    if opts.http_auth:
        auth = opts.http_auth.split(':',1)
    else:
        auth = None
    upload = False
    vars = {}
    for x in (opts.var or ()):
        n,v = x.split('=',1)
        vars[n] = v
    for x in (opts.upload or ()):
        n,v = x.split('=',1)
        upload = True
        vars[n] = TestUpload(v)
    if opts.form_data:
        assert opts.method != 'GET'
        assert not vars
        if opts.form_data == '-':
            stdin = sys.stdin.read()
        else:
            stdin = open(opts.form_data,'rb').read()
        if opts.method == 'POST':
            env, inp = test_environ(url, auth=auth, stdin=stdin)
        else:
            env, inp = test_environ(
                url, env={'CONTENT_TYPE':'multipart/form-data'},
                auth=auth, stdin=stdin
            )
    else:
        if opts.method == 'GET':
            env, inp = test_environ(url, auth=auth, get=vars)
        elif opts.method == 'POST':
            env, inp = test_environ(url, auth=auth, post=vars)
        else:
            env, inp = test_environ(
                url, env={'CONTENT_TYPE':'multipart/form-data'},
                auth=auth, post=vars
            )
    sane_environ(env)
    env['APP_NAME'] = opts.app_name
    return env



class Handler(handlers.SimpleHandler):

    def __init__(self, environ, stdin=None, stdout=None, stderr=None):
        if stdin is None:
            stdin = environ['wsgi.input']
        if stdout is None:
            stdout = StringIO()
        if stderr is None:
            stderr = StringIO()
        handlers.SimpleHandler.__init__(
            self,
            stdin = stdin,
            stdout = stdout,
            stderr = stderr,
            environ = environ,
            multithread = False,
            multiprocess=True,
        )


def handle_url(opts, u):
    env = make_environ(opts, u)
    wsgi = Handler(env, stderr=sys.stderr)
    if opts.throw_errors:
        def handle_error():
            raise
        wsgi.handle_error = handle_error
    wsgi.run(opts.app)
    return wsgi


def print_rusage(rusage, out=sys.stderr):
    import time, socket
    def E(data):
        data = data.replace("&", "&amp;")
        data = data.replace(">", "&gt;")
        data = data.replace("<", "&lt;")
        return data
    out.write('<?xml version="1.0"?>\n')
    out.write('<rusage>\n')
    out.write('  <meta>\n')
    out.write('    <application>%s</application>\n'% opts.app_name)
    out.write('    <created>%s</created>\n' %  time.asctime(time.localtime()))
    out.write('    <host>%s</host>\n' %  socket.getfqdn())
    out.write('    <urls>\n')
    for u in opts.urls:
        out.write('      <url>%s</url>\n' % E(u))
    out.write('    </urls>\n')
    out.write('    <concurrency mode="%s">%d</concurrency>\n'% (
        opts.concurrency_model, opts.concurrency))
    out.write('    <iterations>%d</iterations>\n'% opts.iterations)
    out.write('    <requests>%d</requests>\n'% (
                        opts.iterations * opts.concurrency * len(opts.urls)))
    out.write('  </meta>\n')
    out.write('  <memory>\n')
    for v in 'size', 'resident', 'share', 'text', 'lib', 'data':
        out.write('    <%s>%s</%s>\n' % (v, rusage[v], v))
    out.write('  </memory>\n')
    out.write('  <time>\n')
    for v in 'real', 'user', 'sys':
        out.write('    <%s>%s</%s>\n' % (v, rusage[v], v))
    out.write('  </time>\n')
    out.write('</rusage>\n')


def run_instance(opts):
    last = opts.iterations - 1
    for i in xrange(opts.iterations):
        for u in opts.urls:
            wsgi = handle_url(opts, u)
            if opts.iterations > 1 or opts.concurrency > 1:
                data = wsgi.stdout.getvalue()
                wsgi.stdout.seek(0)
                print >>sys.stderr, '%-3d %s status=%r, size=%r' % (
                    i+1, u, wsgi.stdout.readline().strip(), len(data)
                )
            if i == last and not opts.no_save:
                wsgi.stdout.seek(0,0)
                if not opts.print_headers:
                    while True:
                        l = wsgi.stdout.readline()
                        if not l.strip():
                            break
                for l in wsgi.stdout:
                    sys.stdout.write(l)
                sys.stdout.flush()
        if opts.debug_memory:
            from lib4.debugutils import debug_memory
            debug_memory(stream=sys.stderr)


def run(opts):
    if opts.rusage or opts.concurrency > 1:
        pids = []
        for i in range(opts.concurrency):
            pid = os.fork()
            if pid == 0:
                run_instance(opts)
                sys.exit(0)
            else:
                pids.append(pid)
        if opts.rusage:
            from lib4.debugutils import rusage
            usage = rusage(pids)
            if opts.rusage == '-':
                out = sys.stderr
            else:
                out = open(opts.rusage, 'w')
            print_rusage(usage[None], out)
            if out != sys.stderr:
                out.close()
    else:
        run_instance(opts)



def main():
    opts = get_opts()
    opts.app = make_wsgi_app(os.environ)
    if opts.throw_errors:
        raise NotImplementedError
        # XXX not much reliable method for setting throw_errors
        a = opts.app
        while a:
            if isinstance(a, web.ErrorMiddleware):
                a.throw_errors = True
            a = getattr(a, 'application', None)
    meth = run
    if opts.profile:
        import profile
        profile.run('meth(opts)', opts.profile)
    else:
        meth(opts)
    if opts.interactive_loop:
        import readline
        from lib4.debugutils import interactive_loop
        interactive_loop()

if __name__ == '__main__':
    main()
