# -*- coding: utf-8 -*-

"""Returns package metadata as JSON"""

import os, sys
from setuptools import Command
from distutils.errors import *
from distutils import log
from pkg_resources import parse_requirements, safe_name, parse_version
from pkg_resources import safe_version, EntryPoint, to_filename


class metadata_json(Command):

    description = __doc__

    user_options = []

    boolean_options = []
    negative_opt = {}

    def initialize_options(self):
        self.egg_name = None
        self.egg_version = None
        self.egg_base = None
        self.egg_info = None
        self.tag_build = None
        self.tag_svn_revision = 0
        self.tag_date = 0
        self.broken_egg_info = False
        self.vtags = ''

    def finalize_options (self):
        self.egg_name = safe_name(self.distribution.get_name())
        self.egg_version = safe_version(
                            self.distribution.get_version() + self.vtags)
        try:
            list(
                parse_requirements('%s==%s' % (self.egg_name, self.egg_version))
            )
        except ValueError:
            raise DistutilsOptionError(
                "Invalid distribution name or version syntax: %s-%s" %
                (self.egg_name,self.egg_version)
            )
        if self.egg_base is None:
            dirs = self.distribution.package_dir
            self.egg_base = (dirs or {}).get('',os.curdir)
        self.ensure_dirname('egg_base')
        self.egg_info = to_filename(self.egg_name)+'.egg-info'
        if self.egg_base != os.curdir:
            self.egg_info = os.path.join(self.egg_base, self.egg_info)
        if '-' in self.egg_name: self.check_broken_egg_info()
        # Set package version for the benefit of dumber commands
        # (e.g. sdist, bdist_wininst, etc.)
        #
        self.distribution.metadata.version = self.egg_version
        # If we bootstrapped around the lack of a PKG-INFO, as might be the
        # case in a fresh checkout, make sure that any special tags get added
        # to the version info
        #
        pd = self.distribution._patched_dist
        if pd is not None and pd.key==self.egg_name.lower():
            pd._version = self.egg_version
            pd._parsed_version = parse_version(self.egg_version)
            self.distribution._patched_dist = None

    def check_broken_egg_info(self):
        bei = self.egg_name+'.egg-info'
        if self.egg_base != os.curdir:
            bei = os.path.join(self.egg_base, bei)
        if os.path.exists(bei):
            log.warn(
                "-"*78+'\n'
                "Note: Your current .egg-info directory has a '-' in its name;"
                '\nthis will not work correctly with "setup.py develop".\n\n'
                'Please rename %s to %s to correct this problem.\n'+'-'*78,
                bei, self.egg_info
            )
            self.broken_egg_info = self.egg_info
            self.egg_info = bei     # make it work for now

    def run(self):
        try:
            import json
        except:
            import simplejson as json
        data = dict(
            egg_base = self.egg_base,
            egg_name = self.egg_name,
            egg_version = self.egg_version,
            egg_info = self.egg_info,
            install_requires = self.distribution.install_requires,
            extras_require = self.distribution.extras_require,
            tests_require = self.distribution.tests_require,
            entry_points = self.distribution.entry_points,
            parsed_requirements = {
                '': [r.project_name for r in parse_requirements(
                                self.distribution.install_requires)],
            },
            namespace_packages = self.distribution.namespace_packages,
            packages = self.distribution.packages,
            package_dir = self.distribution.package_dir,
        )
        json.dump(data, sys.stdout, indent=2, sort_keys=True)
        sys.stdout.write('\n')
        #print dir(self.distribution)
        #ep = EntryPoint.parse(self.distribution.entry_points)
        #print ep.__dict__
