# -*- coding: utf-8 -*-

import sys
import os


def proctitle_factory(app, global_config, **settings):
    try:
        import procname
        import itertools
        counter = itertools.count(1).next
    except ImportError:
        def proctitle_middleware_deactivated(environ, start_response):
            return app(environ, start_response)
        return proctitle_middleware_deactivated
    else:
        appname = env['APP_NAME']
        progname = os.path.split(sys.argv[0])[1]
        pad = ' ' * 1000

        procname.setprocname('%s[%s] req=0%s' % (progname, appname, pad))

        def proctitle_middleware(environ, start_response):
            environ['APP_NAME'] = appname
            environ['vinnetou.reqno'] = reqno = counter()
            try:
                if procname:
                    procname.setprocname(
                        '%s[%s] req=%d ra=%s@%s url=%s:%s%s%s' % (
                        progname, appname, reqno,
                        environ.get('REMOTE_USER',''),
                        environ.get('REMOTE_ADDR'),
                        environ.get('HTTP_HOST'),
                        environ.get('SERVER_PORT','80'),
                        environ.get('REQUEST_URI', '/'),
                        pad
                    ))
                return app(environ, start_response)
            finally:
                if procname:
                    procname.setprocname('%s[%s] req=%d%s' % (
                        progname, appname, reqno, pad))
        return proctitle_middleware


def setenv_factory(app, global_config, **settings):

    def setenv_middleware(environ, start_response):
        if 'HTTP_X_FORWARDED_HOST' in environ:
            environ['HTTP_HOST'] = environ['HTTP_X_FORWARDED_HOST']
            #environ['SERVER_NAME'] = environ['HTTP_X_FORWARDED_HOST']
        if 'HTTP_X_FORWARDED_FOR' in environ:
            for a in environ['HTTP_X_FORWARDED_FOR'].split(','):
                a = a.strip()
                if a:
                    environ['REMOTE_ADDR'] = a
                    break
        if 'HTTP_X_FORWARDED_PROTO' in environ:
            proto = environ['HTTP_X_FORWARDED_PROTO']
            environ['wsgi.url_scheme'] = proto
            if proto == 'http':
                environ['SERVER_PORT'] = '80'
                environ['HTTPS'] = 'off'
            elif proto == 'https':
                environ['SERVER_PORT'] = '443'
                environ['HTTPS'] = 'on'
        return app(environ, start_response)
    return setenv_middleware


_max_log_size = 16 * 1024 * 1024
_max_log_backup_files = 1000
_log_format_template = u'%(asctime)s %(levelname)s [%(name)s]: %(message)s'


def logging_factory(app, global_config, **settings):
    import logging
    import logging.handlers
    log_level = getattr(logging, settings.get('level', 'DEBUG').upper())
    log_path = settings.get('file', '/tmp/vinnetou.log')
    handler = logging.handlers.RotatingFileHandler(
            log_path,
            mode="a",
            maxBytes=_max_log_size,
            backupCount=_max_log_backup_files,
            encoding="utf-8"
    )
    formatter = logging.Formatter(_log_format_template)
    handler.setFormatter(formatter)
    logging.root.addHandler(handler)
    logging.root.setLevel(log_level)
    return app
