#!/usr/bin/env python
# -*- coding: ISO-8859-2 -*-

"""
Konfiguracni soubor:

    [appname]
    port=4000    ;port je povinny
    children=42  ;nepovinny
    scgid=path   ;nepovinne, cesta k scgid spousteci
    env=A=1,B=2  ;environment predany scgid. Hodi se pro PYTHONMODE=tag050
"""


from ConfigParser import ConfigParser
from optparse import OptionParser
import subprocess



def do_action(opts):
    def runSCGI(appname):
        ol = opts.config.options(appname)
        port = opts.config.getint(appname, 'port')
        if 'scgid' in ol:
            cmdline = [opts.config.get(appname, 'scgid')]
        else:
            cmdline = [opts.scgid_binary]
        cmdline.append('--' + opts.action)
        cmdline.append('--port=%s' % port)
        if 'children' in ol:
            cmdline.append('--children=%s' % (
                                opts.config.getint(appname, 'children')))
            cmdline.append('--maxspare=%s' % (
                                opts.config.getint(appname, 'children')))
        if 'rlimit-as' in ol:
            cmdline.append('--rlimit-as=%s' % (
                                opts.config.getint(appname, 'rlimit-as')))

        cmdline.append(appname)

        kw = {}
        if 'env' in ol:
            kw['env'] = dict([(r[0], r[1])
                              for r in [e.split('=')
                                        for e in opts.config.get(
                                            appname, 'env').split(',')]])

        print "Running '%s'..." % ' '.join(cmdline),
        print (subprocess.call(cmdline, **kw)==0) and 'OK' or 'FAILED'

    dfl = opts.config.defaults()
    if opts.appname:
        runSCGI(opts.appname)
    else:
        for appname in opts.config.sections():
            runSCGI(appname)



def get_opts():
    actions = ['start', 'stop', 'restart']
    usage = 'usage: %%prog [options] %s [appname]' % ('|'.join(actions))
    parser = OptionParser(usage=usage)

    parser.add_option('--config-file',
                      default='/etc/scgid.conf',
                      metavar='PATH',
                      dest='config_file',
                      help='[default: %default]')
    parser.add_option('--scgid',
                      default='/home/app/windmill4/bin/scgid',
                      metavar='PATH',
                      dest='scgid_binary',
                      help='path to SCGI daemon [default: %default]')

    opts, args = parser.parse_args()
    if len(args) < 1:
        parser.error('action required')
    else:
        action = args[0]
        if action not in actions:
            parser.error("Unknown action '%s'. Only %s actions allowed" % (
                                action,
                                '/'.join(actions)))
        opts.action = action
        if len(args) > 1:
            opts.appname = args[1]
        else:
            opts.appname = None
    opts.config = ConfigParser()
    opts.config.readfp(open(opts.config_file, 'r'))
    return opts


def main():
    opts = get_opts()
    do_action(opts)


if __name__ == '__main__':
    main()
